<?php

/**
 * @file
 * Set of unified functions to process comments for all versions of Drupal.
 *
 */
 
 /**
 * Find a number of recent comments. This is done in two steps.
 *   1. Find the n (specified by $number) nodes that have the most recent
 *      comments.  This is done by querying node_comment_statistics which has
 *      an index on last_comment_timestamp, and is thus a fast query.
 *   2. Loading the information from the comments table based on the nids found
 *      in step 1.
 *
 * @param $number (optional) The maximum number of comments to find.
 * @return $comments An array of comment objects each containing a nid,
 *   subject, cid, and timstamp, or an empty array if there are no recent
 *   comments visible to the current user.
 */

function com2smf_get_recent($number = 10) {
  // Select the $number nodes (visible to the current user) with the most
  // recent comments. This is efficient due to the index on
  // last_comment_timestamp.

  $result = db_query_range("SELECT nc.nid FROM {node_comment_statistics} nc, {smf_thread} smf,". _smf_prefix(). "messages p WHERE nc.comment_count > 0 AND smf.dr_nid = nc.nid AND p.id_topic = smf.smf_thread ORDER BY p.postertime", 0, 0, $number);

  $nids = array();
  while ($row = db_fetch_object($result)) {
    $nids[] = $row->nid;
  }

  $comments = array();
  if (!empty($nids)) {
    // From among the comments on the nodes selected in the first query,
    // find the $number most recent comments.
	$result = db_query('SELECT smf_thread FROM {smf_thread} WHERE dr_nid in ('. implode(',', $nids) .')');

	$threads = array();
	while ($row = db_fetch_object($result)) {
	  $threads[] = $row->smf_thread;
	}
    $result = db_query_range('SELECT smf.dr_nid AS nid, p.subject, p.id_msg AS cid, p.postertime AS timestamp FROM ' ._smf_prefix() .'messages p INNER JOIN {smf_thread} smf ON smf.smf_thread = p.id_topic WHERE p.id_topic IN ('. implode(',', $threads) .') AND p.spam = %d ORDER BY p.id_msg DESC', COMMENT_PUBLISHED, 0, $number);
    while ($comment = db_fetch_object($result)) {
      $comments[] = $comment;
    }
  }
  return $comments;
}

/**
 * Implementation of hook_perm().
 */
function com2smf_perm() {
  return array('access comments', 'post comments', 'administer comments', 'post comments without approval', 'manage users creation in forum');
}

/**
 * Implementation of hook_link().
 */
function com2smf_link($type, $node = NULL, $teaser = FALSE) {
  $links = array();

  if ($type == 'node' && $node->comment) {

    if ($teaser) {
      // Main page: display the number of comments that have been posted.

      if (user_access('access comments')) {
        $all = com2smf_num_all($node->nid);

        if ($all) {
          $links['comment_comments'] = array(
            'title' => format_plural($all, '1 comment', '@count comments'),
            'href' => "node/$node->nid",
            'attributes' => array('title' => t('Jump to the first comment of this posting.')),
            'fragment' => 'comments'
          );

          $new = com2smf_num_new($node->nid);

          if ($new) {
            $links['comment_new_comments'] = array(
              'title' => format_plural($new, '1 new comment', '@count new comments'),
              'href' => "node/$node->nid",
              'attributes' => array('title' => t('Jump to the first new comment of this posting.')),
              'fragment' => 'new'
            );
          }
        }
        else {
          if ($node->comment == COMMENT_NODE_READ_WRITE) {
            if (user_access('post comments')) {
              $links['comment_add'] = array(
                'title' => t('Add new comment'),
                'href' => "comment/reply/$node->nid",
                'attributes' => array('title' => t('Add a new comment to this page.')),
                'fragment' => 'comment-form'
              );
            }
            else {
              $links['comment_forbidden']['title'] = theme('com2smf_post_forbidden', $node->nid);
            }
          }
        }
      }
    }
    else {
      // Node page: add a "post comment" link if the user is allowed to
      // post comments, if this node is not read-only, and if the comment form isn't already shown

      if ($node->comment == COMMENT_NODE_READ_WRITE) {
        if (user_access('post comments')) {
          if (variable_get('comment_form_location', COMMENT_FORM_SEPARATE_PAGE) == COMMENT_FORM_SEPARATE_PAGE) {
            $links['comment_add'] = array(
              'title' => t('Add new comment'),
              'href' => "comment/reply/$node->nid",
              'attributes' => array('title' => t('Share your thoughts and opinions related to this posting.')),
              'fragment' => 'comment-form'
            );
          }
        }
        else {
          $links['comment_forbidden']['title'] = theme('com2smf_post_forbidden', $node->nid);
        }
      }
    }
  }

  if ($type == 'comment') {
    $links = com2smf_links($node, $teaser);
  }
  if (isset($links['comment_forbidden'])) {
    $links['comment_forbidden']['html'] = TRUE;
  }

  return $links;
}

/**
 * Implementation of hook_nodeapi().
 *
 */
function com2smf_nodeapi(&$node, $op, $arg = 0) {
  switch ($op) {
    case 'load':
      return db_fetch_array(db_query("SELECT p.postertime AS last_comment_timestamp, p.postername AS last_comment_name, t.numreplies AS comment_count FROM " ._smf_prefix() ."topics t, " ._smf_prefix() ."messages p, {smf_thread} dr WHERE dr.dr_nid = %d AND t.id_topic = dr.smf_thread AND p.id_msg = t.id_last_msg", $node->nid));
      break;

    case 'prepare':
      if (!isset($node->comment)) {
        $node->comment = variable_get("comment_$node->type", COMMENT_NODE_READ_WRITE);
      }
      break;

    case 'insert':
      db_query('INSERT INTO {node_comment_statistics} (nid, last_comment_timestamp, last_comment_name, last_comment_uid, comment_count) VALUES (%d, %d, NULL, %d, 0)', $node->nid, $node->changed, $node->uid);
	  _smf_start_forum($node, 0);
	  _smf_start_post($node);
      break;
	  
	case 'update':
	  $forum_id = db_result(db_query('SELECT forum_id FROM {smf_thread} WHERE dr_nid = %d', $node->nid));
	  _smf_start_forum($node, $forum_id);
	  break;
	  
	case 'delete':
		$node->smf_thread = db_result(db_query('SELECT smf_thread FROM {smf_thread} WHERE dr_nid = %d', $node->nid));
		$node->cid = $node->smf_thread;
		$op = _com2smf_forum_on_action();
		db_query('DELETE FROM {smf_thread} WHERE dr_nid = %d', $node->nid);
		_com2smf_delete_thread($node, $op);
		break;

    case 'update index':
      $text = '';
      $comments = db_query('SELECT p.subject, p.body AS comment, 1 AS format FROM ' ._smf_prefix() .'messages p, {smf_thread} smf WHERE smf.dr_nid = %d AND p.id_topic = smf.smf_thread AND p.spam = %d', $node->nid, COMMENT_PUBLISHED);

      while ($comment = db_fetch_object($comments)) {
        $text .= '<h2>'. check_plain($comment->subject) .'</h2>'. check_markup($comment->comment, $comment->format, FALSE);
      }
      return $text;

    case 'search result':
      $comments = db_result(db_query('SELECT t.numreplies AS comment_count FROM ' ._smf_prefix() .'topics t, {smf_thread} smf WHERE smf.dr_nid = %d AND t.id_topic = smf.smf_thread', $node->nid));

      return format_plural($comments, '1 comment', '@count comments');

    case 'rss item':
      if ($node->comment != COMMENT_NODE_DISABLED) {
        return array(array('key' => 'comments', 'value' => url('node/'. $node->nid, NULL, 'comments', TRUE)));
      }
      else {
        return array();
      }
  }
}

/**
 * Implementation of hook_user().
 */
function com2smf_user($op, $edit, &$account, $category = NULL) {
  global $user;
  $access = user_access('manage users creation in forum');
  $smfuser = db_result(db_query('SELECT smf_userid FROM {smf_users} WHERE dr_userid = %d', $account->uid));
  
  switch ($op) {
  case 'form':
  case 'register':
    	$form['comment_settings'] = array(
      	'#type' => 'fieldset',
      	'#title' => t('Comment settings'),
      	'#collapsible' => TRUE,
      	'#weight' => 4);
		
		$form['comment_settings']['newsmfuser'] = array(
		'#type' => 'checkbox',
		'#default_value' => variable_get('com2smf_new_user_in_forum', COM2SMF_NEW_USER_IN_FORUM),
		'#title' => 'Create the same user in forum',
		'#description' => t('Users with <em>manage users creation in forum</em> permission can override this settings'),
		'#disabled' => TRUE,
		);

  	if ($category == 'account' && $account->uid == $user->uid) {
    // when user tries to edit his own data
    	$form['comment_settings']['signature'] = array(
      	'#type' => 'textarea',
      	'#title' => t('Signature'),
      	'#default_value' => $edit['signature'],
      	'#description' => t('Your signature will be publicly displayed at the end of your comments.'));
	}
		
	if (!$smfuser && $access) {
		$form['comment_settings']['newsmfuser']['#disabled'] = FALSE;
	}
   	return $form;
  	
  case 'delete':
    db_query('UPDATE {comments} SET uid = 0 WHERE uid = %d', $account->uid);
	db_query('UPDATE {node_comment_statistics} SET last_comment_uid = 0 WHERE last_comment_uid = %d', $account->uid);
//	$smfuser = db_result(db_query('SELECT smf_userid FROM {smf_users} WHERE dr_userid = %d', $user->uid));
	db_query('UPDATE ' ._smf_prefix() .'messages SET id_member = 0 WHERE id_member = %d', $smfuser);
    db_query('UPDATE ' ._smf_prefix() .'topics SET id_last_msg = 0 WHERE id_last_msg = %d', $smfuser);
	db_query('DELETE FROM {smf_users} WHERE dr_userid = %d AND smf_userid = %d', $account->uid, $smfuser);
  break;
  
  case 'insert':
  	if ($edit['newsmfuser']) {
		$smfuserid = _com2smf_user_create($account, $edit);
		if ($smfuserid > 0) {
			$text = t('User was created in forum with Id = ').$smfuserid;
			drupal_set_message($text, 'status');
			watchdog('user', $text, WATCHDOG_NOTICE);
		}
		elseif ($smfuserid == 0) {
			$text = t('Unknown error while creating a user in forum');
			drupal_set_message($text, 'error');
			watchdog('user', $text, WATCHDOG_ERROR);
		}
		elseif ($smfuserid < 0) {
			$text = t('The name %name is used in forum.', array('%name' => $edit['name']));
			drupal_set_message($text, 'error');
			watchdog('user', $text, WATCHDOG_ERROR);
		}
	}
  break;
  
  case 'validate':
  	if (variable_get('com2smf_new_user_check', COM2SMF_NEW_USER_CHECK)) {
	_com2smf_user_validate($account, $edit, $smfuser);
	}
  break;
  
  case 'update':
	if (!$smfuser && $edit['newsmfuser']) {
		com2smf_user('insert', $edit, $account, $category);
		break;
	}
  	$result = _com2smf_user_update($account, $edit, $smfuser);
	if ($result) {
		$text = t('User %name was updated in forum', array('%name' => $edit['name']));
		drupal_set_message($text, 'status');
		watchdog('user', $text, WATCHDOG_NOTICE);
	}
	else {
		$text = t('User %name was not updated in forum', array('%name' => $edit['name']));
		drupal_set_message($text, 'error');
		watchdog('user', $text, WATCHDOG_ERROR);
	}
  break;
  
  case 'login':
  	if (_com2smf_user_login($account)) {
		drupal_set_message(t('You are successfully loged to forum'), 'status');
		watchdog('user', t('User %user loged to forum.', array('%user' => $account->name)), WATCHDOG_NOTICE);
	}
	else {
		drupal_set_message(t('You are not loged to forum or banned in forum'), 'error');
		watchdog('user', t('Login to forum failed for user %user.', array('%user' => $account->name)), WATCHDOG_ERROR);
	}
  break;
  
  case 'logout':
  	_com2smf_user_logout($account);
  }
}

/**
 * This is *not* a hook_access() implementation. This function is called
 * to determine whether the current user has access to a particular comment.
 *
 * Authenticated users can edit their comments as long they have not been
 * replied to. This prevents people from changing or revising their
 * statements based on the replies to their posts.
 */
function com2smf_access($op, $comment) {
  global $user;

  if ($op == 'edit') {
    return ($user->uid && $user->uid == $comment->uid && com2smf_num_replies($comment->cid) == 0) || user_access('administer comments');
  }
}

/**
 *
 */
function com2smf_node_url() {
  return arg(0) .'/'. arg(1);
}

/**
 * Accepts a submission of new or changed comment content.
 *
 * @param $edit
 *   A comment array.
 *
 * @return
 *   If the comment is successfully saved the comment ID is returned. If the comment
 *   is not saved, FALSE is returned.
 */
function com2smf_save($edit) {

  global $user, $base_url, $node;
  
// converting drupal's user to SMF's user
  $edit['uid'] = db_result(db_query("SELECT smf_userid AS uid FROM {smf_users} WHERE dr_userid = %d", $edit['uid']));
  //checking mode of posting: real posting or moving of old comments from drupal
  if ($edit['export'] == 0) $edit['ipaddress'] = _com2smf_ip_address();
  
  if (user_access('post comments') && (user_access('administer comments') || node_comment_mode($edit['nid']) == COMMENT_NODE_READ_WRITE)) {
    if (!form_get_errors()) {
      if ($edit['cid']) {
        // Update the comment in the database.
        _com2smf_update_node_statistics($edit['nid']);

        // Allow modules to respond to the updating of a comment.
        com2smf_invoke_comment($edit, 'update');

        // Add an entry to the watchdog log.
        watchdog('content', t('Comment: updated %subject.', array('%subject' => $edit['subject'])), WATCHDOG_NOTICE, l(t('view'), 'node/'. $edit['nid'], array('fragment' => 'comment-'. $edit['cid'])));

      }
      else {
        // Check for duplicate comments. Note that we have to use the
        // validated/filtered data to perform such check.

        $duplicate = db_result(db_query("SELECT COUNT(p.id_msg) FROM " ._smf_prefix() ."messages p, {smf_thread} smf WHERE smf.dr_nid = %d AND p.id_topic = smf.smf_thread AND p.subject = '%s' AND p.body = '%s'", $edit['nid'], $edit['subject'], $edit['comment']), 0);
        if ($duplicate != 0) {
          watchdog('content', t('Comment: duplicate %subject.', array('%subject' => $edit['subject'])), WATCHDOG_WARNING);
        }

        // Add the comment to database.
        $edit['status'] = user_access('post comments without approval') ? COMMENT_PUBLISHED : COMMENT_NOT_PUBLISHED;
        $roles = variable_get('comment_roles', array());
        $score = 0;

        foreach (array_intersect(array_keys($roles), array_keys($user->roles)) as $rid) {
          $score = max($roles[$rid], $score);
        }

        // Here we are building the thread field. See the documentation for
        // comment_render().
        if ($edit['pid'] == 0) {
          // This is a comment with no parent comment (depth 0): we start
          // by retrieving the maximum thread level.
          //finding id of start post in forum
		$forum_set = db_fetch_object(db_query('SELECT t.id_first_msg, t.id_board FROM {smf_thread} smf, ' ._smf_prefix() .'topics t WHERE smf.dr_nid = %d AND t.id_topic = smf.smf_thread', $edit['nid']));
		$edit['pid'] = $forum_set->id_first_msg;
		$edit['id_board'] = $forum_set->id_board;
          //if start post is absence, than we create it.
		  if ($edit['pid'] == 0) {
		  	_com2smf_save($edit, 1);
		  }
        }
		if (variable_get('com2smf_html_conv', COM2SMF_HTML_CONV)) {
		  $edit['comment'] = html2bbcode($edit['comment']);
		  }
		  
        $edit['smf_thread'] = db_result(db_query("SELECT smf_thread FROM {smf_thread} WHERE dr_nid = %d", $edit['nid']));
		$forum_set = db_fetch_object(db_query('SELECT t.id_first_msg, t.id_board FROM {smf_thread} smf, ' ._smf_prefix() .'topics t WHERE smf.dr_nid = %d AND t.id_topic = smf.smf_thread', $edit['nid']));
		$edit['pid'] = $forum_set->id_first_msg;
		$edit['id_board'] = $forum_set->id_board;
        _com2smf_save($edit, 0);
		// increasing post counter of SMF's user
		if ($edit['uid'] > 0) db_query("UPDATE " ._smf_prefix() ."members SET posts = posts+1 WHERE id_member = %d", $edit['uid']);
		// founding id of newest comment and updating comments statistic in SMF
        $edit['cid'] = db_result(db_query("SELECT MAX(LAST_INSERT_ID(id_msg)) FROM " ._smf_prefix() ."messages"));
		// Increasing reply's counter of thread
		db_query("UPDATE " ._smf_prefix() ."topics SET numreplies = numreplies + 1 WHERE id_topic = %d", $edit['smf_thread'], $edit['timestamp']);
		// Updating information about last post in thread
		db_query("UPDATE " ._smf_prefix() ."topics SET id_last_msg = %d, id_member_updated = %d WHERE id_topic = %d AND id_last_msg < %d", $edit['cid'], $edit['uid'], $edit['smf_thread'], $edit['cid']);
		// updating stastic of board in SMF
		$thread_req = db_fetch_object(db_query("SELECT t.id_board AS forumid, p.subject AS title FROM " ._smf_prefix() ."topics t,". _smf_prefix(). "messages p WHERE t.id_topic = %d AND p.id_msg = t.id_first_msg", $edit['smf_thread']));
		$edit['smf_forumid'] = $thread_req->forumid;
		$edit['smf_threadname'] = $thread_req->title;
		// increasing count of replays
		db_query("UPDATE " ._smf_prefix() ."boards SET numposts = numposts + 1 WHERE id_board = %d", $edit['smf_forumid']);
		// updating time of last post if need
		db_query("UPDATE " ._smf_prefix() ."boards SET id_last_msg = %d WHERE id_last_msg < %d AND id_board = %d", $edit['cid'], $edit['cid'], $edit['smf_forumid']);

// copying comment in "native" table {comments} for backward compatibility with other modules, which working with table {comments} directly (Notify, Akismet, etc.)
        db_query("INSERT INTO {comments} (cid, nid, pid, uid, subject, format, hostname, timestamp, status, thread, name, mail, homepage) VALUES (%d, %d, %d, %d, '%s', %d, '%s', %d, %d, '%s', '%s', '%s', '%s')", $edit['cid'], $edit['nid'], $edit['pid'], $edit['uid'], $edit['subject'], $edit['format'], $edit['ipaddress'], $edit['timestamp'], $edit['status'], $thread, $edit['name'], $edit['mail'], $edit['homepage']);

        _com2smf_update_node_statistics($edit['nid']);

        // Tell the other modules a new comment has been submitted.
        com2smf_invoke_comment($edit, 'insert');

        // Add an entry to the watchdog log.
        watchdog('content', t('Comment: added %subject.', array('%subject' => $edit['subject'])), WATCHDOG_NOTICE, l(t('view'), 'node/'. $edit['nid'], array('fragment' => 'comment-'. $edit['cid'])));
      }

      // Clear the cache so an anonymous user can see his comment being added.
      cache_clear_all();

      // Explain the approval queue if necessary, and then
      // redirect the user to the node he's commenting on.
      if ($edit['status'] == COMMENT_NOT_PUBLISHED) {
        drupal_set_message(t('Your comment has been queued for moderation by site administrators and will be published after approval.'));
      }
      return $edit['cid'];
    }
    else {
      return FALSE;
    }
  }
  else {
    $txt = t('Comment: unauthorized comment submitted or comment submitted to a closed node %subject.', array('%subject' => $edit['subject']));
    watchdog('content', $txt, WATCHDOG_WARNING);
    drupal_set_message($txt, 'error');
    return FALSE;
  }
}

function com2smf_links($comment, $return = 1) {
  global $user;

  $links = array();

  // If we are viewing just this comment, we link back to the node.
  if ($return) {
    $links['comment_parent'] = array(
      'title' => t('parent'),
      'href' => com2smf_node_url(),
      'fragment' => "comment-$comment->cid"
    );
  }

  if (node_comment_mode($comment->nid) == COMMENT_NODE_READ_WRITE) {
    if (user_access('administer comments') && user_access('post comments')) {
/*
      $links['comment_delete'] = array(
        'title' => t('delete'),
        'href' => "comment/delete/$comment->cid"
      );

      $links['comment_edit'] = array(
        'title' => t('edit'),
        'href' => "comment/edit/$comment->cid"
      );
*/
      $links['comment_reply'] = array(
        'title' => t('reply'),
        'href' => "comment/reply/$comment->nid/$comment->cid"
      );
	  
      $links['comment_host'] = array(
        'title' => t('Host: ' . $comment->hostname),
      );

      $links['comment_mail'] = array(
        'title' => t('E-mail: ' . $comment->mail),
      );

    }
    else if (user_access('post comments')) {
      if (com2smf_access('edit', $comment)) {
        $links['comment_edit'] = array(
          'title' => t('edit'),
          'href' => "comment/edit/$comment->cid"
        );
      }
      $links['comment_reply'] = array(
        'title' => t('reply'),
        'href' => "comment/reply/$comment->nid/$comment->cid"
      );
    }
    else {
      $links['comment_forbidden']['title'] = theme('com2smf_post_forbidden', $comment->nid);
    }
  }

  return $links;
}

/**
 * Menu callback; delete a comment.
 */
function com2smf_delete($cid = NULL) {
  $comment = db_fetch_object(db_query('SELECT c.*, u.name AS registered_name, u.uid FROM {comments} c INNER JOIN {users} u ON u.uid = c.uid WHERE c.cid = %d', $cid));
  $comment->name = $comment->uid ? $comment->registered_name : $comment->name;

  $output = '';

  if (is_object($comment) && is_numeric($comment->cid)) {
    $output = drupal_get_form('com2smf_confirm_delete', $comment);
  }
  else {
    drupal_set_message(t('The comment no longer exists.'));
  }

  return $output;
}

/**
 * Stub for backward compatibility with Akismet module
 */
function comment_operations($action = NULL) {
	return com2smf_operations($action);
}
 
/**
 * Comment operations. We offer different update operations depending on
 * which comment administration page we're on.
 */
function com2smf_operations($action = NULL) {
  if ($action == 'publish') {
    $operations = array(
      'publish' => array(t('Publish the selected comments'), 'UPDATE ' ._smf_prefix() .'messages SET spam = '. COMMENT_PUBLISHED .' WHERE id_msg = %d')
    );
  }
  else if ($action == 'unpublish') {
    $operations = array(
      'unpublish' => array(t('Unpublish the selected comments'), 'UPDATE ' ._smf_prefix() .'messages SET spam = '. COMMENT_NOT_PUBLISHED .' WHERE id_msg = %d')
    );
  }
  else {
    $operations = array(
      'publish' => array(t('Publish the selected comments'), 'UPDATE ' ._smf_prefix() .'messages SET spam = '. COMMENT_PUBLISHED .' WHERE id_msg = %d'),
      'unpublish' => array(t('Unpublish the selected comments'), 'UPDATE ' ._smf_prefix() .'messages SET spam = '. COMMENT_NOT_PUBLISHED .' WHERE id_msg = %d')
    );
  }
  return $operations;
}

/**
*** misc functions: helpers, privates, history
**/

/**
 * Load the entire comment by cid.
 */
function _com2smf_load($cid) {
  return db_fetch_object(db_query('SELECT * FROM ' ._smf_prefix() .'messages WHERE id_msg = %d', $cid));
}

/*
 *
 */
function com2smf_num_all($nid) {
  static $cache;

  if (!isset($cache[$nid])) {
    $cache[$nid] = db_result(db_query('SELECT t.numreplies AS comment_count FROM {smf_thread} smf, ' ._smf_prefix() .'topics t WHERE smf.dr_nid = %d AND t.id_topic = smf.smf_thread', $nid));
  }
  return $cache[$nid];
}

/*
 *
 */
function com2smf_num_replies($pid) {
  static $cache;

  if (!isset($cache[$pid])) {
    $cache[$pid] = db_result(db_query('SELECT COUNT(cid) FROM {comments} WHERE pid = %d AND status = %d', $pid, COMMENT_PUBLISHED));
  }

  return $cache[$pid];
}

/**
 * get number of new comments for current user and specified node
 *
 * @param $nid node-id to count comments for
 * @param $timestamp time to count from (defaults to time of last user access
 *   to node)
 */
function com2smf_num_new($nid, $timestamp = 0) {
  global $user;

  if ($user->uid) {
    // Retrieve the timestamp at which the current user last viewed the
    // specified node.
    if (!$timestamp) {
      $timestamp = node_last_viewed($nid);
    }
    $timestamp = ($timestamp > NODE_NEW_LIMIT ? $timestamp : NODE_NEW_LIMIT);

    // Use the timestamp to retrieve the number of new comments.
    $result = db_result(db_query('SELECT COUNT(p.id_msg) FROM {node} n LEFT JOIN {smf_thread} smft ON smft.dr_nid = n.nid LEFT JOIN ' ._smf_prefix() .'messages p ON p.id_topic = smft.smf_thread WHERE n.nid = %d AND p.postertime > %d AND p.spam = %d', $nid, $timestamp, COMMENT_PUBLISHED));

    return $result;
  }
  else {
    return 0;
  }

}

function com2smf_validate($edit) {
  global $user;

  // Invoke other validation handlers
  com2smf_invoke_comment($edit, 'validate');

  if (isset($edit['date'])) {
    // As of PHP 5.1.0, strtotime returns FALSE upon failure instead of -1.
    if (strtotime($edit['date']) <= 0) {
      form_set_error('date', t('You have to specify a valid date.'));
    }
  }
  if (isset($edit['author']) && !$account = user_load(array('name' => $edit['author']))) {
    form_set_error('author', t('You have to specify a valid author.'));
  }

  // Check validity of name, mail and homepage (if given)
  if (!$user->uid || isset($edit['is_anonymous'])) {
    if (variable_get('comment_anonymous', COMMENT_ANONYMOUS_MAYNOT_CONTACT) > COMMENT_ANONYMOUS_MAYNOT_CONTACT) {
      if ($edit['name']) {
        $taken = db_result(db_query("SELECT COUNT(uid) FROM {users} WHERE LOWER(name) = '%s'", $edit['name']), 0);

        if ($taken != 0) {
          form_set_error('name', t('The name you used belongs to a registered user.'));
        }

      }
      else if (variable_get('comment_anonymous', COMMENT_ANONYMOUS_MAYNOT_CONTACT) == COMMENT_ANONYMOUS_MUST_CONTACT) {
        form_set_error('name', t('You have to leave your name.'));
      }

      if ($edit['mail']) {
        if (!valid_email_address($edit['mail'])) {
          form_set_error('mail', t('The e-mail address you specified is not valid.'));
        }
      }
      else if (variable_get('comment_anonymous', COMMENT_ANONYMOUS_MAYNOT_CONTACT) == COMMENT_ANONYMOUS_MUST_CONTACT) {
        form_set_error('mail', t('You have to leave an e-mail address.'));
      }

      if ($edit['homepage']) {
        if (!valid_url($edit['homepage'], TRUE)) {
          form_set_error('homepage', t('The URL of your homepage is not valid. Remember that it must be fully qualified, i.e. of the form <code>http://example.com/directory</code>.'));
        }
      }
    }
  }

  return $edit;
}

/**
 * Stub for backward compatibility with Akismet module
 */
function _comment_delete_thread($comment, $op = 'nothing') {
	_com2smf_delete_thread($comment, $op);
}

/**
 * Processing of deleting a comment or thread. Action depends on 'com2smf_forum_on_action' variable
 */
function _com2smf_delete_thread($comment, $op = 'nothing') {
  if (!is_object($comment) || !is_numeric($comment->cid)) {
    watchdog('content', t('Can not delete non-existent comment.'), WATCHDOG_WARNING);
    return;
  }
  
  if ($comment->smf_thread != '') {
  	$op .= '_smf';
	}
	
	//watchdog('com2smf', t('_com2smf_delete_thread: $op = %op', array('%op' => $op)));
	//watchdog('com2smf', t('_com2smf_delete_thread: $comment->smf_thread = %op', array('%op' => $comment->smf_thread)));
  
	switch($op) {
		case 'nothing':
			// Delete the comment from {comments} and link to a forum's thread:
			db_query('DELETE FROM {comments} WHERE cid = %d', $comment->cid);
			//db_query('DELETE FROM ' ._smf_prefix() .'post WHERE postid = %d', $comment->cid);
			watchdog('content', t('Comment: removed from Drupal %subject.', array('%subject' => $comment->subject)));
			break;
		
		case 'unpublish':
			db_query('UPDATE ' ._smf_prefix() .'post SET visible = 0 WHERE postid = %d', $comment->cid);
			break;
			
		case 'delete':
			// Delete the comment from {comments} and link to a forum's thread:
			db_query('DELETE FROM {comments} WHERE cid = %d', $comment->cid);
			db_query('DELETE FROM ' ._smf_prefix() .'messages WHERE id_msg = %d', $comment->cid);
			db_query('UPDATE ' ._smf_prefix() .'members SET posts = posts - 1 WHERE id_member = %d', $comment->userid);
			watchdog('content', t('Comment: deleted %subject.', array('%subject' => $comment->subject)));
			break;
		
		case 'nothing_smf':
			$smf = db_query('SELECT id_first_msg FROM ' ._smf_prefix() .'topics WHERE id_topic = %d', $comment->smf_thread);
//***SMF later***			$comment = db_fetch_object(db_query('SELECT id_msg AS cid FROM ' ._smf_prefix() .'messages WHERE parentid = %d', $smf));
			$op = 'nothing';
			return;
		
		case 'unpublish_smf':
			db_query("UPDATE " ._smf_prefix() ."topics SET spam = 0 AND hidden_reason = 'Unpublished from Drupal' WHERE id_topic = %d", $comment->smf_thread);
			return;
			//break;
		
		case 'close_smf':
			db_query('UPDATE ' ._smf_prefix() .'topics SET locked = 1 WHERE id_topic = %d', $comment->smf_thread);
			return;
			//break;
		
		case 'delete_smf':
			$smf = db_result(db_query('SELECT id_first_msg FROM ' ._smf_prefix() .'topics WHERE id_topic = %d', $comment->smf_thread));
			db_query('DELETE FROM ' ._smf_prefix() .'topics WHERE id_topic = %d', $comment->smf_thread);
			//db_query('DELETE FROM ' ._smf_prefix() .'post WHERE postid = %d', $smf);
			//$comment = db_fetch_object(db_query('SELECT postid as cid FROM ' ._smf_prefix() .'post WHERE parentid = %d', $smf));
			$op = 'delete';
			//return;
		}

	com2smf_invoke_comment($comment, 'delete');

	// Delete the comment's replies
	$result = db_query('SELECT p.id_msg AS cid, p.id_topic, p.id_member, p.subject, p.postertime, u.name AS registered_name, u.uid FROM ' ._smf_prefix() .'messages p LEFT JOIN {smf_users} smfu ON smfu.smf_userid = p.id_member LEFT JOIN {users} u ON u.uid = smfu.dr_userid WHERE p.id_topic = %d', $comment->smf_thread);
	while ($comment = db_fetch_object($result)) {
    $comment->name = $comment->uid ? $comment->registered_name : $comment->name;
    _com2smf_delete_thread($comment, $op);
  }
}

/**
 * Return an array of viewing modes for comment listings.
 *
 * We can't use a global variable array because the locale system
 * is not initialized yet when the comment module is loaded.
 */
function _com2smf_get_modes() {
  return array(
    COMMENT_MODE_FLAT_COLLAPSED => t('Flat list - collapsed'),
    COMMENT_MODE_FLAT_EXPANDED => t('Flat list - expanded'),
    COMMENT_MODE_THREADED_COLLAPSED => t('Threaded list - collapsed')
  );
}

/**
 * Return an array of viewing orders for comment listings.
 *
 * We can't use a global variable array because the locale system
 * is not initialized yet when the comment module is loaded.
 */
function _com2smf_get_orders() {
  return array(
    COMMENT_ORDER_NEWEST_FIRST => t('Date - newest first'),
    COMMENT_ORDER_OLDEST_FIRST => t('Date - oldest first')
  );
}

/**
 * Return an array of "comments per page" settings from which the user
 * can choose.
 */
function _com2smf_per_page() {
  return drupal_map_assoc(array(1, 2, 3, 5, 10, 20, 100));
}

/**
 * Return a current comment display setting
 *
 * $setting can be one of these: 'mode', 'sort', 'comments_per_page'
 */
function _com2smf_get_display_setting($setting, $node = NULL) {
  global $user;
  
  if (!isset($node)) {
  	$node->type = '';
  }
  else {
  	$node->type = '_'. $node->type;
	}

  if (isset($_GET[$setting])) {
    $value = $_GET[$setting];
  }
  else {
    // get the setting's site default
    switch ($setting) {
      case 'mode':
        $default = variable_get('comment_default_mode'. $node->type, COMMENT_MODE_FLAT_EXPANDED);
        break;
      case 'sort':
        $default = variable_get('comment_default_order'. $node->type, COMMENT_ORDER_NEWEST_FIRST);
        break;
      case 'comments_per_page':
        $default = variable_get('comment_default_per_page'. $node->type, '5');
    }
    if (variable_get('comment_controls'. $node->type, COMMENT_CONTROLS_HIDDEN) == COMMENT_CONTROLS_HIDDEN) {
      // if comment controls are disabled use site default
      $value = $default;
    }
    else {
      // otherwise use the user's setting if set
      if ($user->$setting) {
        $value = $user->$setting;
      }
      else if ($_SESSION['comment_'. $setting]) {
        $value = $_SESSION['comment_'. $setting];
      }
      else {
        $value = $default;
      }
    }
  }
  return $value;
}

/**
 * Updates the comment statistics for a given node. This should be called any
 * time a comment is added, deleted, or updated.
 *
 * The following fields are contained in the node_comment_statistics table.
 * - last_comment_timestamp: the timestamp of the last comment for this node or the node create stamp if no comments exist for the node.
 * - last_comment_name: the name of the anonymous poster for the last comment
 * - last_comment_uid: the uid of the poster for the last comment for this node or the node authors uid if no comments exists for the node.
 * - comment_count: the total number of approved/published comments on this node.
 */
function _com2smf_update_node_statistics($nid) {
  $count = db_result(db_query('SELECT COUNT(p.id_msg) AS cid FROM ' ._smf_prefix() .'messages p, {smf_thread} smf WHERE smf.dr_nid = %d AND p.id_topic = smf.smf_thread AND p.spam = %d', $nid, COMMENT_PUBLISHED));

  // comments exist
  if ($count > 0) {
    $last_reply = db_fetch_object(db_query_range('SELECT p.id_msg AS cid, p.postername AS name, p.postertime AS timestamp, u.dr_userid AS uid FROM ' ._smf_prefix() .'messages p, {smf_users} u, {smf_thread} smf  WHERE smf.dr_nid = %d AND p.id_topic = smf.smf_thread AND p.spam = %d ORDER BY p.id_msg DESC', $nid, COMMENT_PUBLISHED, 0, 1));
    db_query("UPDATE {node_comment_statistics} SET comment_count = %d, last_comment_timestamp = %d, last_comment_name = '%s', last_comment_uid = %d WHERE nid = %d", $count, $last_reply->timestamp, $last_reply->uid ? '' : $last_reply->name, $last_reply->uid, $nid);
  }

  // no comments
  else {
    $node = db_fetch_object(db_query("SELECT uid, created FROM {node} WHERE nid = %d", $nid));
    db_query("UPDATE {node_comment_statistics} SET comment_count = 0, last_comment_timestamp = %d, last_comment_name = '', last_comment_uid = %d WHERE nid = %d", $node->created, $node->uid, $nid);
  }
}

/**
 * Stub for backward compatibility with Akismet module
 */
function _comment_update_node_statistics($nid) {
	_com2smf_update_node_statistics($nid);
}

/**
 * Stub for backward compatibility with Akismet module
 */
function comment_invoke_comment(&$comment, $op) {
	switch($op) {
		case 'delete':
		db_query('DELETE FROM' ._smf_prefix() .'messages WHERE id_msg = %d', $comment->cid);
		break;
		
		case 'unpublish':
		db_query("UPDATE " ._smf_prefix() ."messages SET spam = 1, hidden_reason = 'Unpublished from Drupal' WHERE id_msg = %d", $comment->cid);
		break;
		
		case 'publish':
		db_query("UPDATE " ._smf_prefix() ."messages SET spam = 0, hidden_reason = 'Published from Drupal' WHERE id_msg = %d", $comment->cid);
	}
	
	watchdog('com2smf', 'Akismet sad "'.$op.'" comment '.$comment->cid, WATCHDOG_WARNING);
}

/**
 * Invoke a hook_comment() operation in all modules.
 *
 * @param &$comment
 *   A comment object.
 * @param $op
 *   A string containing the name of the comment operation.
 * @return
 *   The returned value of the invoked hooks.
 */
function com2smf_invoke_comment(&$comment, $op) {
  $return = array();
  foreach (module_implements('comment') as $name) {
    $function = $name .'_comment';
    $result = $function($comment, $op);
    if (isset($result) && is_array($result)) {
      $return = array_merge($return, $result);
    }
    else if (isset($result)) {
      $return[] = $result;
    }
  }
  return $return;
}

/**
 * Get a list of forums in SMF database
 * @return
 * Associated array containing forums list
 */
function _smf_forum_list() {
	
	$forum_list = array();
		$result = db_query('SELECT cat.name AS groupname, fr.id_board, fr.name AS forumname FROM ' ._smf_prefix() .'categories cat, ' ._smf_prefix() .'boards fr WHERE fr.id_cat = cat.id_cat ORDER BY cat.catorder, fr.boardorder');
		while ($values = db_fetch_object($result)) {
			$forum_list[$values->groupname][$values->id_board] = $values->forumname;
	}
	
	return $forum_list;
}

/**
 * Get current settings of action on deleting a node
 * @return
 * Id of action
 */
function _com2smf_forum_on_action() {
	return variable_get('com2smf_forum_on_action', COM2SMF_FORUM_DEFAULT_ACTION);
}

/**
 * Ancillary function to com2smf_save()
 * If $start = 1, then we are creating a start post
 */
function _com2smf_save($edit, $start = 0) {
	global $base_url;
	
	if ($start) {
		//taking a "preview" of active version of node for inserting in forum
		$dr_teaser = db_fetch_array(db_query('SELECT v.teaser, v.title, n.uid FROM {node_revisions} v, {node} n WHERE n.nid = %d and v.vid = n.vid', $edit['nid']));
		$dr_teaser['teaser'] = node_teaser($dr_teaser['teaser']);
		//converting html tags to bbcode
		if (variable_get('com2smf_html_conv', COM2SMF_HTML_CONV)) {
			$dr_teaser['teaser'] = html2bbcode($dr_teaser['teaser']);
		}
		//taking a URL to node
		$dr_teaser['link'] = $base_url. COM2SMF_DR_NODE_TEMP. $edit['nid'];
		//formating body of start post in forum
		$fr_body = "[QUOTE]". $dr_teaser['teaser']. "[/QUOTE][URL=". $dr_teaser['link']."]". t('Read more'). "[/URL]";
		//define the forum to post comment:
		//checking for individual settings for node
		$edit['id_board'] = db_result(db_query('SELECT forum_id FROM {smf_thread} WHERE dr_nid = %d', $edit['nid']));
		$forum_init = 1;
		//what is default forum to post for this node type?
		if (!$edit['id_board']) {
			$ntype = 'com2smf_type_' . db_result(db_query('SELECT type FROM {node} WHERE nid = %d', $edit['nid']));
			$edit['id_board'] = variable_get($ntype, _smf_forum_id());
			$forum_init = 0;
		}
		
		$edit['subject'] =  $dr_teaser['title'];
		$edit['comment'] = $fr_body;
		if ($edit['timestamp'] == 0) $edit['timestamp'] = time();
		}

		// Insert post in forum's database		
		db_query("INSERT INTO " ._smf_prefix() ."messages (id_topic, id_member, id_board, subject, body, posterip, postertime, spam, postername, smileysenabled, posteremail) VALUES (%d, %d, %d, '%s', '%s', '%s', %d, %d, '%s', %d, '%s')", $edit['smf_thread'], $edit['uid'], $edit['id_board'], $edit['subject'], $edit['comment'], $edit['ipaddress'], $edit['timestamp'] - 1, COMMENT_PUBLISHED, $edit['name'], 1, $edit['mail']);
		
		if ($start) {
			$edit['pid'] = db_result(db_query("SELECT MAX(LAST_INSERT_ID(id_msg)) FROM " ._smf_prefix() ."messages"));
			db_query("INSERT INTO " ._smf_prefix() ."topics (id_first_msg, id_last_msg, id_board, locked, id_member_started, id_member_updated, numreplies) VALUES(%d, %d, %d, 0, %d, '%s', 0)", $edit['pid'], $edit['pid'], $edit['id_board'], $edit['uid'], $edit['uid']);
			$edit['smf_thread'] = db_result(db_query("SELECT MAX(LAST_INSERT_ID(id_topic)) FROM " ._smf_prefix() ."topics"));
			if ($forum_init == 0) {
				db_query('INSERT INTO {smf_thread} (dr_nid, smf_thread) VALUES (%d, %d)', $edit['nid'], $edit['smf_thread']);
			}
			else {
				db_query('UPDATE {smf_thread} SET smf_thread = %d WHERE dr_nid = %d', $edit['smf_thread'], $edit['nid']);
			}
			db_query("UPDATE " ._smf_prefix() ."messages SET id_topic = %d WHERE id_msg = %d", $edit['smf_thread'], $edit['pid']);
			// updating statistic of board in SMF
			// increasing count of replays
			db_query("UPDATE " ._smf_prefix() ."boards SET numposts = numposts + 1, numtopics = numtopics + 1 WHERE id_board = %d", $edit['id_board']);
		}
}

/**
 * Get current settings of forum to post
 * @return
 * Associated array containing form of forums list
 */
function _smf_forum_select($var = NULL, $multiple = FALSE) {

	$def_forum = _smf_forum_id();
	if (isset($var['node_type'])) {
		$def_node_forum = variable_get('com2smf_type_' . $var['node_type'], $def_forum);
		}
		else {
		$def_node_forum = $def_forum;
	}

	return array(
    '#type' => 'select',
	'#title' => t('Default forum to post comments'),
	'#default_value' => $def_node_forum,
	'#options' => _smf_forum_list(),
	'#size' => 5,
	'#maxlength' => 16,
	'#multiple' => $multiple,
  );
}

/**
 * Get a board in SMF to post comments
 * @return
 * Id of forum as integer
 */
function _smf_forum_id() {
	return variable_get('com2smf_def_forum', COM2SMF_FORUM_POST);
}
	
/**
 * Get a prefix of SMF's database
 * @return
 * String containing prefix
 */
function _smf_prefix() {
	return variable_get('com2smf_db_prefix', COM2SMF_PREFIX);
}

/**
 * Remove specific to SMF extensions of bbcode for correct displaying in Drupal
 */
function _smf_clear_bbcode($text) {
	// [ur="..."] -> [ur=...]
	$text = preg_replace('#(\[url[^"]*?)"([^"]*?)"([^"]*?url\])#i', '$1$2$3', $text);
	// [quote author=Anonimous link=topic=290.msg1311#msg1311 date=1272708907] -> [quote=Anonimous]
	$text = preg_replace('#(\[quote)( author)([^ link]*?)([^\]]*?)(\])#i', '$1$3$5', $text);
	// <br /> -> /r
	$text = str_replace('<br />', chr(10), $text);
	
	return $text;
 }

function _smf_start_forum($node, $forum_id = 0) {
	if (!$node->com2smf_forum_chk) {
	  	if ($forum_id > 0) :
	  		db_query('UPDATE {smf_thread} SET forum_id = %d WHERE dr_nid = %d', $node->com2smf_forum, $node->nid);
		 	else :
		 		db_query('INSERT INTO {smf_thread} (dr_nid, forum_id) VALUES (%d, %d)', $node->nid, $node->com2smf_forum);
		 	endif;
		}
		elseif ($forum_id > 0) {
			db_query('DELETE FROM {smf_thread} WHERE dr_nid = %d', $node->nid);
		}
}

function _smf_start_post($node) {
	if ($node->status && $node->com2smf_start_post) {
		$edit = array();
		
		$edit['smf_thread'] = db_result(db_query('SELECT smf_thread FROM {smf_thread} WHERE dr_nid = %d', $node->nid));
		$edit['nid'] = $node->nid;
		$edit['uid'] = $node->uid;
		$edit['name'] = db_result(db_query('SELECT name FROM {users} WHERE uid = %d', $edit['uid']));
		//Who will be the author of start post?
		switch (variable_get('com2smf_teaser_author', COM2SMF_TEASER_AUTHOR)) {
			case 0:
				$edit['uid'] = db_result(db_query('SELECT smf_userid FROM {smf_users} WHERE dr_userid = %d', $node->uid));
			break;
			
			default:
				$edit['uid'] = 0;
			break;
		}
		_com2smf_save($edit, 1);
		}
}

/*
 * Backport of function ip_address() from Drupal 6
 */
function _com2smf_ip_address() {

  static $ip_address = NULL;

  if (!isset($ip_address)) {
    $ip_address = $_SERVER['REMOTE_ADDR'];
    if (variable_get('reverse_proxy', 0) && array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)) {
      // If an array of known reverse proxy IPs is provided, then trust
      // the XFF header if request really comes from one of them.
      $reverse_proxy_addresses = variable_get('reverse_proxy_addresses', array());
      if (!empty($reverse_proxy_addresses) && in_array($ip_address, $reverse_proxy_addresses, TRUE)) {
        // If there are several arguments, we need to check the most
        // recently added one, i.e. the last one.
        $ip_address = array_pop(explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']));
      }
    }
  }

  return $ip_address;

}